unit ucuBuscarEmprestimos;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, acuFramework;

type
  ucBuscarEmprestimos = class(ucUseCase)
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  published
    procedure Effect_BuscarEmprestimos(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage {define utField},
  utuEmprestimoOQL,
  acuObject {define acEnumerator},
  nguExemplar,

  nguLivro,

  nguSituacaoExemplar,

  nguEmprestimo,

  nguCliente,

  dateUtils;

procedure ucBuscarEmprestimos.Initialize;
begin
  inherited;
end;

procedure ucBuscarEmprestimos.Finalize;
begin
  
  inherited;
end;

procedure ucBuscarEmprestimos.Effect_BuscarEmprestimos(piRequest: utRequest; piTransition: utTransition);
var
lFieldEmprestimos,
lFieldEmprestimo,
lFieldExemplar,
lFieldLivro,
lFieldCliente: utField;
lEmprestimo: ngEmprestimo;
lExemplar: ngExemplar;
lLivro: ngLivro;
lCliente: ngCliente;
lOQL: IObterEmprestimos;
lListaEmprestimos: acPersistentObjectList;
lEnumEmprestimo: acEnumerator;
begin
  {Pegamo input e criamos root field do xml resposta}
  lFieldCliente := piRequest.Message.RootField.FieldByName('Cliente');
  lFieldEmprestimos:= piRequest.Response.RootField.AddField('Emprestimos');

  lOQL := utEmprestimoOQL.ObterEmprestimos(Self.Session);

  lListaEmprestimos := acPersistentObjectList.Create(ngEmprestimo);
  try
    {Setamos os parmetros da busca e ento a chamamos}
    if lFieldCliente.HasAttribute('Nome')
      then lOQL.Param_nome.Value := lFieldCliente.AttributeByName('Nome').AsString
      else lOQL.Param_nome.SetNull;

    if lFieldCliente.HasAttribute('CPF')
      then lOQL.Param_cpf.Value := lFieldCliente.AttributeByName('CPF').AsString
      else lOQL.Param_cpf.SetNull;

    lOQL.LoadList(lListaEmprestimos);

    {Percorremos a lista resposta da consulta ao banco e escrevemos os campos da nossa resposta}
    lEnumEmprestimo := acEnumerator.Create(lListaEmprestimos);
    try
      while not lEnumEmprestimo.EOL do
      begin
        lEmprestimo := lEnumEmprestimo.Current as ngEmprestimo;
        lExemplar := lEmprestimo.Exemplar.Partner as ngExemplar;
        lLivro := lExemplar.Livro.Partner as ngLivro;
        lCliente := lEmprestimo.Cliente.Partner as ngCliente;

        lFieldEmprestimo := lFieldEmprestimos.AddField('Emprestimo');
        lFieldCliente := lFieldEmprestimo.AddField('Cliente');
        lFieldExemplar := lFieldEmprestimo.AddField('Exemplar');
        lFieldLivro := lFieldExemplar.AddField('Livro');

        lEmprestimo.toXML(lFieldEmprestimo);
        lCliente.toXML(lFieldCliente);
        lExemplar.toXML(lFieldExemplar);
        lLivro.toXML(lFieldLivro);

        lEnumEmprestimo.MoveNext;
      end;
    finally
      lEnumEmprestimo.Free;
    end;

  finally
    lListaEmprestimos.Free;
  end;
  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3223,ucBuscarEmprestimos);

end.
